@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2018 by the individuals mentioned in the source code history
+#   Copyright 2007-2019 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -35,7 +35,8 @@
 setClass(Class = "MxBaseExpectation", 
 	 representation = representation(
 		 data = "MxCharOrNumber",      # filled in during flattening
-		 dataColumns = "integer",      # subset and permutation of data columns
+		 dataColumns = "integer",      # deprecated
+		 dataColumnNames = "character",  # subset and permutation of data columns
 	     .runDims = "character",
 	     output = "list",
 	     debug = "list",
