@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2018 by the individuals mentioned in the source code history
+#   Copyright 2007-2019 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -46,14 +46,16 @@ setClass(Class = "MxExpectationLISREL",
 		thresholds = "MxCharOrNumber",
 		dims = "character",
 		threshnames = "character",
-		depth = "integer"), #Used to speed up I-A inverse in RAM, could be used to speed up I-B inverse in LISREL
+		depth = "integer", # speed up I-A inverse in RAM; speed up I-B inverse in LISREL
+		verbose = "integer"),
 	contains = "BaseExpectationNormal")
 
 
 #--------------------------------------------------------------------
 # **DONE**
 setMethod("initialize", "MxExpectationLISREL",
-	function(.Object, LX, LY, BE, GA, PH, PS, TD, TE, TH, TX, TY, KA, AL, dims, thresholds, threshnames,
+	function(.Object, LX, LY, BE, GA, PH, PS, TD, TE, TH, TX, TY, KA, AL,
+		 dims, thresholds, threshnames, verbose,
 		data = as.integer(NA), name = 'expectation') {
 		.Object@name <- name
 		.Object@LX <- LX
@@ -69,6 +71,7 @@ setMethod("initialize", "MxExpectationLISREL",
 		.Object@TY <- TY
 		.Object@KA <- KA
 		.Object@AL <- AL
+		.Object@verbose <- verbose
 		.Object@data <- data
 		.Object@dims <- dims
 		.Object@thresholds <- thresholds
@@ -158,15 +161,7 @@ checkLISRELmeansHelper <- function(Lam, Mean, Latent, matrixname, lamname, model
 		checkStg <- "row"
 	}
 	# Check that the means are non-null
-	if(is.null(Mean)){
-		msg <- paste("The LISREL expectation function",
-			"has an observed means vector but",
-			"is missing expected means vector",
-			"for", varType, "variables",
-			matrixname,  "in model",
-			omxQuotes(modelname))
-		stop(msg, call. = FALSE)
-	}
+	if(is.null(Mean)) return()
 	# Check that the means have dimnames
 	meanDimnames <- dimnames(Mean)
 	if(is.null(meanDimnames)){
@@ -452,9 +447,9 @@ setMethod("genericExpFunConvert", signature("MxExpectationLISREL"),
 		#  Set the canonical order of observed variable names.
 		translatedNames <- c(dimnames(lyMatrix)[[1]], dimnames(lxMatrix)[[1]]) #fMatrixTranslateNames(fMatrix, modelname) #Rearrange the rownames of F to match the order of the columns
 		.Object@depth <- generateLISRELDepth(flatModel, beMatrix2, model@options) #Find out how many iterations of I + BE + BE^2 + ... are need until nilpotency.
-		if (mxDataObject@type == 'raw' || mxDataObject@type == 'acov') {
+		if (dataIsRawish(mxDataObject)) {
 			threshName <- .Object@thresholds
-			checkNumberOrdinalColumns(mxDataObject)
+			.Object@dataColumnNames <- translatedNames
 			.Object@dataColumns <- generateDataColumns(flatModel, translatedNames, data)
 			verifyThresholds(flatModel, model, labelsData, data, translatedNames, threshName)
 			.Object@thresholds <- imxLocateIndex(flatModel, threshName, name)
@@ -534,7 +529,7 @@ checkLISRELargument <- function(x, xname) {
 
 #--------------------------------------------------------------------
 # **DONE**
-mxExpectationLISREL <- function(LX=NA, LY=NA, BE=NA, GA=NA, PH=NA, PS=NA, TD=NA, TE=NA, TH=NA, TX = NA, TY = NA, KA = NA, AL = NA, dimnames = NA, thresholds = NA, threshnames = dimnames) {
+mxExpectationLISREL <- function(LX=NA, LY=NA, BE=NA, GA=NA, PH=NA, PS=NA, TD=NA, TE=NA, TH=NA, TX = NA, TY = NA, KA = NA, AL = NA, dimnames = NA, thresholds = NA, threshnames = dimnames, verbose=0L) {
 	LX <- checkLISRELargument(LX, "LX")
 	LY <- checkLISRELargument(LY, "LY")
 	BE <- checkLISRELargument(BE, "BE")
@@ -564,7 +559,7 @@ mxExpectationLISREL <- function(LX=NA, LY=NA, BE=NA, GA=NA, PH=NA, PS=NA, TD=NA,
 		stop("NA values are not allowed for dimnames vector")
 	}
 	threshnames <- checkThreshnames(threshnames)
-	return(new("MxExpectationLISREL", LX, LY, BE, GA, PH, PS, TD, TE, TH, TX, TY, KA, AL, dimnames, thresholds, threshnames))
+	return(new("MxExpectationLISREL", LX, LY, BE, GA, PH, PS, TD, TE, TH, TX, TY, KA, AL, dimnames, thresholds, threshnames, as.integer(verbose)))
 }
 
 
