@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2018 by the individuals mentioned in the source code history
+#   Copyright 2007-2019 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -95,7 +95,7 @@ mxTryHard <- function(model, extraTries = 10, greenOK = FALSE, loc = 1,
 	model@compute@.persist <- TRUE
 	modelAtStartValues <- suppressWarnings(try(mxRun(model, suppressWarnings = T, unsafe=T, silent=T, intervals=FALSE)))
 	if(class(modelAtStartValues) != "try-error"){ 
-		fitvalAtStarts <- modelAtStartValues@fitfunction@result[1,1]
+		fitvalAtStarts <- modelAtStartValues@fitfunction@result[1]
 		if(is.finite(fitvalAtStarts)){lowestminsofar <- fitvalAtStarts}
 	}
 	model@compute <- inputCompute
@@ -190,7 +190,7 @@ mxTryHard <- function(model, extraTries = 10, greenOK = FALSE, loc = 1,
 		
 		
 		#If fit resulted in error:
-		if( class(fit) == "try-error" || !is.finite(fit@fitfunction@result[1,1]) || fit$output$status$status== -1){
+		if( class(fit) == "try-error" || !is.finite(fit@fitfunction@result[1]) || fit$output$status$status== -1){
 			#^^^is.finite() returns FALSE for Inf, -Inf, NA, and NaN
 			lastBestFitCount <- 0
 			lastNoError<-FALSE
@@ -200,15 +200,15 @@ mxTryHard <- function(model, extraTries = 10, greenOK = FALSE, loc = 1,
 		
 		
 		#If fit did NOT result in error:
-		if(class(fit) != "try-error" && is.finite(fit@fitfunction@result[1,1]) && fit$output$status$status != -1){
+		if(class(fit) != "try-error" && is.finite(fit@fitfunction@result[1]) && fit$output$status$status != -1){
 			lastNoError <- TRUE
 			validcount <- validcount + 1
-			if(fit@fitfunction@result[1,1] >= lowestminsofar){
+			if(fit@fitfunction@result[1] >= lowestminsofar){
 				lastBestFitCount <- 0
-				if(fit@fitfunction@result[1,1] >= lowestminsofar + generalTolerance){
-					if(!silent){message(paste0('\n Fit attempt worse than current best:  ',fit@fitfunction@result[1,1] ,' vs ', lowestminsofar ))}
+				if(fit@fitfunction@result[1] >= lowestminsofar + generalTolerance){
+					if(!silent){message(paste0('\n Fit attempt worse than current best:  ',fit@fitfunction@result[1] ,' vs ', lowestminsofar ))}
 					else{
-						msg <- paste0('Fit attempt ',numdone-1,', fit=',fit@fitfunction@result[1,1],', worse than previous best (',lowestminsofar,')')
+						msg <- paste0('Fit attempt ',numdone-1,', fit=',fit@fitfunction@result[1],', worse than previous best (',lowestminsofar,')')
 						imxReportProgress(msg, previousLen)
 						previousLen <- nchar(msg)
 					}
@@ -216,19 +216,19 @@ mxTryHard <- function(model, extraTries = 10, greenOK = FALSE, loc = 1,
 			#Current fit will become bestfit if (1) its fitvalue is strictly less than lowestminsofar, or
 			#(2) its fitvalue is no greater than lowestminsofar (within tolerance) AND it satisfies the criteria for 
 			#an acceptable result (i.e., goodflag gets set to TRUE):
-			if(fit@fitfunction@result[1,1] < lowestminsofar){ #<--If this is the best fit so far
-				if(!silent){message(paste0('\n Lowest minimum so far:  ',fit@fitfunction@result[1,1]))}
+			if(fit@fitfunction@result[1] < lowestminsofar){ #<--If this is the best fit so far
+				if(!silent){message(paste0('\n Lowest minimum so far:  ',fit@fitfunction@result[1]))}
 				else{
-					msg <- paste0('Fit attempt ',numdone-1,', fit=',fit@fitfunction@result[1,1],', new current best! (was ',lowestminsofar,')')
+					msg <- paste0('Fit attempt ',numdone-1,', fit=',fit@fitfunction@result[1],', new current best! (was ',lowestminsofar,')')
 					imxReportProgress(msg, previousLen)
 					previousLen <- nchar(msg)
 				}
 				lastBestFitCount<-lastBestFitCount+1 
-				lowestminsofar <- fit@fitfunction@result[1,1]
+				lowestminsofar <- fit@fitfunction@result[1]
 				bestfit <- fit
 				bestfit.params <- omxGetParameters(bestfit)
 			}
-			if(fit@fitfunction@result[1,1] <= lowestminsofar + generalTolerance){
+			if(fit@fitfunction@result[1] <= lowestminsofar + generalTolerance){
 				###########goodflag checks
 				goodflag <- TRUE
 				if( !(fit$output$status[[1]] %in% OKstatuscodes) ){
@@ -238,8 +238,8 @@ mxTryHard <- function(model, extraTries = 10, greenOK = FALSE, loc = 1,
 													paste("(",paste(OKstatuscodes,collapse=","),")",sep=""), sep=""))
 					}
 				}
-				if(fit@fitfunction@result[1,1] > fit2beat) {
-					if(!silent){message(paste0('\n Fit value of ', fit@fitfunction@result[1,1], ' greater than fit2beat of ', fit2beat))}
+				if(fit@fitfunction@result[1] > fit2beat) {
+					if(!silent){message(paste0('\n Fit value of ', fit@fitfunction@result[1], ' greater than fit2beat of ', fit2beat))}
 					goodflag <- FALSE
 				}
 				if(checkHess==TRUE) {
@@ -268,7 +268,7 @@ mxTryHard <- function(model, extraTries = 10, greenOK = FALSE, loc = 1,
 			if(iterationSummary){
 				message(paste0("\n Attempt ",numdone-1," result:  "))
 				message(paste(names(params),": ", fit$output$estimate,"\n"))
-				message(paste0("fit value = ", fit@fitfunction@result[1,1]))
+				message(paste0("fit value = ", fit@fitfunction@result[1]))
 			}
 		} #end 'if fit did not result in error' section
 		
@@ -319,13 +319,13 @@ mxTryHard <- function(model, extraTries = 10, greenOK = FALSE, loc = 1,
 			}
 		}
 		imxReportProgress("", previousLen)
-		message(paste0("\nSolution found!  Final fit=", signif(bestfit@fitfunction@result[1,1],8), " (started at ", signif(fitvalAtStarts,8), ")  (" ,numdone, " attempt(s): ", validcount, " valid, ", errorcount," errors)\n"))
+		message(paste0("\nSolution found!  Final fit=", signif(bestfit@fitfunction@result[1],8), " (started at ", signif(fitvalAtStarts,8), ")  (" ,numdone, " attempt(s): ", validcount, " valid, ", errorcount," errors)\n"))
 		if (length(summary(bestfit)$npsolMessage) > 0) {
 			warning(summary(bestfit)$npsolMessage)
 		}
 		if(iterationSummary){
 			message(paste(names(bestfit.params),": ", bestfit$output$estimate,"\n"))
-			message(paste0("fit value = ", bestfit@fitfunction@result[1,1]))
+			message(paste0("fit value = ", bestfit@fitfunction@result[1]))
 		}
 		bestfit <- THFrankenmodel(finalfit,bestfit,defaultComputePlan,Hesslater,SElater,doIntervals,checkHess)
 	} #end 'if goodflag' section
@@ -368,7 +368,7 @@ mxTryHard <- function(model, extraTries = 10, greenOK = FALSE, loc = 1,
 				}
 			}
 			imxReportProgress("", previousLen)
-			message(paste0("\nRetry limit reached; solution not found.  Best fit=", signif(bestfit@fitfunction@result[1,1],8), " (started at ", signif(fitvalAtStarts,8), ")  (", numdone, " attempt(s): ", validcount, " valid, ", errorcount," errors)\n"))
+			message(paste0("\nRetry limit reached; solution not found.  Best fit=", signif(bestfit@fitfunction@result[1],8), " (started at ", signif(fitvalAtStarts,8), ")  (", numdone, " attempt(s): ", validcount, " valid, ", errorcount," errors)\n"))
 			if (length(bestfit$output$status$statusMsg) > 0) { 
 				warning(bestfit$output$status$statusMsg)
 			}
@@ -377,7 +377,7 @@ mxTryHard <- function(model, extraTries = 10, greenOK = FALSE, loc = 1,
 			}
 			if(iterationSummary){
 				message(paste(names(bestfit.params),": ", bestfit$output$estimate,"\n"))
-				message(paste0("fit value = ", bestfit@fitfunction@result[1,1]))
+				message(paste0("fit value = ", bestfit@fitfunction@result[1]))
 			}
 			bestfit <- THFrankenmodel(finalfit,bestfit,defaultComputePlan,Hesslater,SElater,doIntervals,checkHess)
 		}
