@@ -1,11 +1,8 @@
 /* Steepest Descent optimizer for unconstrained problems*/
 
+#include "ComputeGD.h"
 #include "ComputeSD.h"
-#include "finiteDifferences.h"
-
-#ifdef SHADOW_DIAG
-#pragma GCC diagnostic warning "-Wshadow"
-#endif
+#include "EnableWarnings.h"
 
 struct fit_functional {
 	GradientOptimizerContext &goc;
@@ -46,9 +43,8 @@ void omxSD(GradientOptimizerContext &rf)
     fit_functional ff(rf);
     Eigen::VectorXd majorEst = currEst;
 
-    while(++iter < maxIter && !isErrorRaised()) {
-	    gradient_with_ref(rf.gradientAlgo, 1, rf.gradientIterations, rf.gradientStepSize,
-			      ff, refFit, majorEst, rf.grad);
+    while(++iter < maxIter && !isErrorRaised() && !Global->timedOut) {
+	    rf.numericalGradientWithRef(majorEst);
 
 	    if (rf.verbose >= 3) mxPrintMat("grad", rf.grad);
 
