@@ -7,10 +7,15 @@
 #ifdef EXTRA_GCC_DIAG
 #pragma GCC diagnostic warning "-Wshadow"
 #pragma GCC diagnostic warning "-Wmisleading-indentation"
+#pragma GCC diagnostic warning "-Wall"
 #pragma GCC diagnostic warning "-Wextra"
+#pragma GCC diagnostic warning "-Wvla"
 #pragma GCC diagnostic warning "-Wunused-function"
 #pragma GCC diagnostic warning "-Wunused-local-typedefs"
 #pragma GCC diagnostic warning "-Wdeprecated-declarations"
 #pragma GCC diagnostic warning "-Wignored-attributes"
 #pragma GCC diagnostic warning "-Wint-in-bool-context"
+#pragma GCC diagnostic warning "-Wstrict-aliasing"
+#pragma GCC diagnostic warning "-Wunused-variable"
+#pragma GCC diagnostic warning "-Wignored-attributes"
 #endif
